/*
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * Netbufs don't come with nifty queuing functions
 * like mbufs.  So make some simple ones that we will
 * use.  We keep stats on what was dropped from
 * which queue.  This is independent from the stats
 * kept for pppstats.  We may not use them, but
 * then again... we may modify later.
 *
 * Steve Perkins, 1994.
 *
 */

#include <stddef.h>
#include <sys/types.h>
#include <net/netbuf.h>


/*
 * Create a queueing mechanism for netbufs.
 *
 */

struct netbuf_item
{
  netbuf_t data;
  struct netbuf_item *next;
};


struct netbuf_queue
{
  unsigned int max_len;
  unsigned int len;
  unsigned int num_dropped;
  struct netbuf_item *head;
  struct netbuf_item *tail;
}; 

void initialize_netbuf_queue(struct netbuf_queue *, u_int max_len);
void free_netbuf_queue(struct netbuf_queue *);
int netbuf_enqueue(netbuf_t, struct netbuf_queue *, int raise);
netbuf_t netbuf_dequeue(struct netbuf_queue *, int raise);
int netbuf_queue_empty_p(struct netbuf_queue *);

