/* 
 * ppp_async.hc - Point-to-Point Protocol (PPP) Asynchronous driver.
 *
 * Copyright (c) 1989 Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by Carnegie Mellon University.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Drew D. Perkins
 * Carnegie Mellon University
 * 4910 Forbes Ave.
 * Pittsburgh, PA 15213
 * (412) 268-8576
 * ddp@andrew.cmu.edu
 *
 * Based on:
 *	@(#)if_sl.c	7.6.1.2 (Berkeley) 2/15/89
 *
 * Copyright (c) 1987 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Serial Line interface
 *
 * Rick Adams
 * Center for Seismic Studies
 * 1300 N 17th Street, Suite 1450
 * Arlington, Virginia 22209
 * (703)276-7900
 * rick@seismo.ARPA
 * seismo!rick
 *
 * Pounded on heavily by Chris Torek (chris@mimsy.umd.edu, umcp-cs!chris).
 * Converted to 4.3BSD Beta by Chris Torek.
 * Other changes made at Berkeley, based in part on code by Kirk Smith.
 *
 * Converted to 4.3BSD+ 386BSD by Brad Parker (brad@cayman.com)
 * Added VJ tcp header compression; more unified ioctls
 *
 * Extensively modified by Paul Mackerras (paulus@cs.anu.edu.au).
 * Cleaned up a lot of the mbuf-related code to fix bugs that
 * caused system crashes and packet corruption.  Changed pppstart
 * so that it doesn't just give up with a collision if the whole
 * packet doesn't fit in the output ring buffer.
 *
 * Added priority queueing for interactive IP packets, following
 * the model of if_sl.c, plus hooks for bpf.
 * Paul Mackerras (paulus@cs.anu.edu.au).
 *
 * Ultrix port by Per Sundstrom <sundstrom@stkhlm.enet.dec.com>,
 * Robert Olsson <robert@robur.slu.se> and Paul Mackerras.
 *
 * NeXT port by Steve Perkins <perkins@cps.msu.edu>.  Based on
 * Ultrix port by Sundstrom, Olsson, and Mackerras.
 * Converted mbuf related code to netbuf_t code.  Switched from 
 * struct ifnet to  netif_t.  Added support for LKS.  Separated
 * asynchronous tty code from LKS code.
 *
 */

/* $Id: ppp_async.h,v 1.1 1994/12/31 01:54:07 perkins Exp $ */
/* from if_sl.c,v 1.11 84/10/04 12:54:47 rick Exp */

#ifndef _PPP_ASYNC_H_
#define _PPP_ASYNC_H_



int     install_ld __P(());
int	pppopen __P((dev_t dev, struct tty *tp));
void	pppclose __P((struct tty *tp));
int	pppread __P((struct tty *tp, struct uio *uio));   /* move received input to user space */
int	pppwrite __P((struct tty *tp, struct uio *uio));  /* Move user space output to kernel and send */
int	ppptioctl __P((struct tty *tp, int cmd, void *data, int flag));
void    ppprend __P((char *cp, u_int n, struct tty *tp));
void	pppinput __P((int c, struct tty *tp));  /* A bottom half routine called async */
void	pppstart __P((struct tty *tp));
int	pppasyncstart __P((struct ppp_softc *));

u_short	pppfcs __P((u_short fcs, u_char *cp, int len));

void	ppp_dump_netbuf __P((netbuf_t, int pktlen));
void	pppdumpb __P((u_char *b, int l));
void	ppplogchar __P((struct ppp_softc *, int));

/*
 * Definitions for system defined functions.
 */

int     ttymodem __P((struct tty* tp, int dcd_on)); 
int     ttselect __P((struct tty *tp, int rw));


#endif /* _PPP_ASYNC_H_ */
